public with sharing class TestPackageManagement {
	
	//there was some issue when using group signer
	public void testPackageManagement(){
		//create package
		String packageId = createPackage();
		
		//retrieve a package
		ESignLiveSDK sdk = new ESignLiveSDK();
		ESignLiveAPIObjects.Package_x pkg = sdk.getPackage(packageId);
		
		//retrieve packages with query parameters
		List<ESignLiveAPIObjects.Package_x> pkgs = getPackages(new Map<String,String>{'from'=>'0','to'=>'9','status'=>'SENT'});
		
		//update package
		ESignLiveAPIObjects.Package_x pkg1 = new ESignLiveAPIObjects.Package_x();
		pkg1.name = 'changed package name';
		sdk.updatePackage(pkg1, packageId);
		
		
		//archive package
		sdk.setStatus(packageId, ESignLiveAPIObjects.PackageStatus.ARCHIVED);	//you can only archive completed packages
		sdk.setStatus(packageId, ESignLiveAPIObjects.PackageStatus.COMPLETED); //restore archive
		
		//trash package
		pkg.trashed = true;    //trash action
		sdk.updatePackage(pkg, packageId);
		
		//delete package
		sdk.deletePackage(packageId); //delete action
		
	}
	
	
	 public String createPackage() {
        //Create esl client
        ESignLiveSDK sdk = new ESignLiveSDK();
         
        //Create package
        ESignLiveAPIObjects.Package_x pkg = new ESignLiveAPIObjects.Package_x();
        pkg.name = 'Test Designer - ' + Datetime.now().format();
         
        String packageId = sdk.createPackage(pkg);
        System.debug('PackageId: ' + packageId);
        
        return packageId;   
    }
    
    
    //new TestPackageManagement().getPackages(new Map<String,String>{'from'=>'6','to'=>'10','status'=>'SENT'});
    //https://docs.esignlive.com/content/c_integrator_s_guide/rest_api/packages.htm#Retrieve
    //for all query parameters
   	public List<ESignLiveAPIObjects.Package_x> getPackages(Map<String,String> queryParameters){
		// Send Request
		String queryString = '';
		if(queryParameters != null && queryParameters.size() > 0){
			List<String> parameters = new List<String>();
			for(String key: queryParameters.keySet()){
				parameters.add(key + '=' + queryParameters.get(key));
			}
			queryString = '?' + String.join(parameters, '&');
		}
        String resourceUrl = '/packages'+ queryString;
        HttpResponse response = doGetWithoutPrepareInboundJSON(resourceUrl);
            	
        // Check response
        if(response.getStatusCode() == 200)
        {
            try
            {
            	Map<String, Object> responseMap = (Map<String, Object>)JSON.deserializeUntyped(response.getBody());
            	List<Object> resultsList = (List<Object>)responseMap.get('results');
            	
            	List<ESignLiveAPIObjects.Package_x> packages = new List<ESignLiveAPIObjects.Package_x>();
            	for(Object packObj: resultsList){
            		String inboundResponseBody = ESignLiveJSONHelper.prepareInboundJSON(JSON.serialize(packObj));
            		ESignLiveAPIObjects.Package_x curPkg = (ESignLiveAPIObjects.Package_x)JSON.deserialize(inboundResponseBody,ESignLiveAPIObjects.Package_x.class);
            		System.debug('package id:' + curPkg);
            		packages.add(curPkg);
            	}
            	return packages;
            }
            catch(Exception e)
            {
                throw new ESignLiveRESTAPIHelper.ESignLiveRestAPIHelperException('Error converting packages json ' + e.getMessage());
            }
        }   
        else
        {
            throw new ESignLiveRESTAPIHelper.ESignLiveRestAPIHelperException('Error get packages ' + response.getStatusCode() + ' - ' + response.getStatus() + ' - ' +  response.getBody());
        }
	}
        
       private HttpResponse doGetWithoutPrepareInboundJSON(String resourceUrl){
        e_SignLive_Connection_Settings__c connectionSettings = e_SignLive_Connection_Settings__c.getInstance('Main');    
         // Build Request
        HttpRequest request = new HttpRequest();
        request.setEndpoint(connectionSettings.Endpoint__c + resourceUrl);
        request.setMethod('GET');
        request.setHeader('Authorization', 'Basic ' + connectionSettings.API_Key__c);
        request.setHeader('Content-type', 'application/json');

        // Send Request
        HttpResponse response = (new Http()).send(request);

        return response;
    }
}